import BaseController from '../common/base.controller';

export class RouterMap {
    static decoratedRouters: Map<Function | Function[], {
        target?: any,
        method?: string,
        path?: string,
        roles?: string[],
        permissions?: string[]
    }> = new Map()
}
export function router(route?: string) {
    return (target: BaseController, name: string, value: PropertyDescriptor) => {
        if (!route) {
            const controller = target.constructor.name
            const controllerName = controller.toLowerCase().replace('.controller', '')
            route = 'all ' + ['', controllerName, name].join('/')
        }
        const split = route.split(' ');
        if (split.length > 2) {
            throw new Error('路由中只允许一个空格')
        }
        const [method, path] = split
        // @ts-ignore
        const key = target[name];
        let routerObj = {
            target: target,
            path: path,
            method: method
        };
        if (RouterMap.decoratedRouters.has(key)) {
            let objCurrent = RouterMap.decoratedRouters.get(key);
            Object.assign(objCurrent, routerObj);
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], objCurrent);
        } else {
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], routerObj);
        }
    }
}
export function role(roles?: string | string[]) {
    return (target: BaseController, name: string, value: PropertyDescriptor) => {
        let roleList: string[] = [];
        if (roles) {
            if (Array.isArray(roles)) {
                roleList = roles;
            } else {
                roleList = [roles];
            }
        }
        // @ts-ignore
        const key = target[name];
        let roleObj = {roles: roleList};
        if (RouterMap.decoratedRouters.has(key)) {
            let objCurrent = RouterMap.decoratedRouters.get(key);
            Object.assign(objCurrent, roleObj);
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], objCurrent);
        } else {
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], roleObj);
        }
    }
}

export function permission(permissions?: string | string[]) {
    return (target: BaseController, name: string, value: PropertyDescriptor) => {
        let permissionList: string[] = [];
        if (permissions) {
            if (Array.isArray(permissions)) {
                permissionList = permissions;
            } else {
                permissionList = [permissions];
            }
        }
        // @ts-ignore
        const key = target[name];
        let permissionObj = {permissions: permissionList};
        if (RouterMap.decoratedRouters.has(key)) {
            let objCurrent = RouterMap.decoratedRouters.get(key);
            Object.assign(objCurrent, permissionObj);
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], objCurrent);
        } else {
            // @ts-ignore
            RouterMap.decoratedRouters.set(target[name], permissionObj);
        }
    }
}
