import {
    getModelForClass,
    index,
    modelOptions,
    prop
} from "@typegoose/typegoose";
import {Base, TimeStamps} from "@typegoose/typegoose/lib/defaultClasses";
import {dbconn} from "../decorators/dbconn";
import {Card} from "./subdoc/Card";

interface CardGroupClass extends Base<string>, TimeStamps {
}

/**
 * 卡组信息
 */

@dbconn()
@index({accountid: 1, heroid: 1, deleted: 1}, {unique: false})
@modelOptions({
    schemaOptions:
        {collection: "card_group", timestamps: true}
})
class CardGroupClass {
    @prop({required: true})
    public accountid!: string;
    @prop({required: true})
    public heroid!: number;
    /**
     * 是否是默认选中的卡组
     */
    @prop({default: false})
    public selected: boolean;
    /**
     * 是否是默认卡组
     */
    @prop({default: false})
    public isdefault: boolean;
    @prop({_id: false, type: () => [Card]})
    public cards: Card[];
    /**
     * 删除标记
     */
    @prop({default: false})
    public deleted: boolean;
    /**
     * 删除时间
     */
    @prop()
    public deletetime: Date;

    public toJson() {
        return {
            gid: this._id,
            heroid: this.heroid,
            selected: this.selected,
            isdefault: this.isdefault,
            cards: this.cards.map(o => o.toJson())
        }
    }
}

export const CardGroup = getModelForClass(CardGroupClass,
// @ts-ignore
    {existingConnection: CardGroupClass['db']})
