import {
    FastifyInstance,
    FastifyPluginAsync, FastifyReply,
    FastifyRequest,
} from 'fastify';
import fastifyPlugin from 'fastify-plugin';
import {User} from '../models/User';
import {ZError} from "../common/ZError";

declare module 'fastify' {
    interface FastifyInstance {
        apiAuth: (request: FastifyRequest, reply: FastifyReply) => {};
    }

    interface FastifyRequest {
        session?: {[key: string]: any};
        roles?: any,
        user?: any
    }
}

const apiAuthPlugin: FastifyPluginAsync = async function(
    fastify: FastifyInstance,
    options?: any
) {

    // 只有路由配置的role为anon才不需要过滤
    fastify.decorate("apiAuth", async function(request: FastifyRequest, reply: FastifyReply) {
        if (!request.roles || request.roles.indexOf('anon') == -1) {
            try {
                // @ts-ignore
                let { accountid, sessionid } = request.params;
                //TODO: 增加sessionid的校验
                // if (!accountid || !sessionid) {
                //     return reply.send({code: 11, msg: 'need accountid and sessionid'});
                // }
                if (!accountid) {
                    return reply.send({code: 2, msg: 'need accountid and sessionid'});
                }
                // const data = this.jwt.verify(request.token);
                // if (!data || !data.id) {
                //     return reply.send({code: 10, msg: 'need login'});
                // }
                let account = await User.findById(accountid);
                if (!account) {
                    return reply.send({code: 5, msg: 'account not found'});
                }
                if (account.locked) {
                    return reply.send({code: 4, msg: 'account locked'});
                }
                request.user = account;
            } catch (err) {
                return reply.send({code: 401, msg: 'need auth'})
            }
        }

    })
};

export = fastifyPlugin(apiAuthPlugin, {
    fastify: '>=3.0.0',
    name: 'apiauth',
});
